package de.ullisroboterseite.ursai2notification;

import java.io.File;
import java.util.*;

import com.google.appinventor.components.annotations.*;
import com.google.appinventor.components.annotations.androidmanifest.*;
import com.google.appinventor.components.common.*;
import com.google.appinventor.components.runtime.*;
import com.google.appinventor.components.runtime.util.*;

import android.app.*;
import android.content.*;
import android.os.Build;
import android.net.Uri;

import androidx.core.content.FileProvider;

import android.util.Log;

// https://developer.android.com/reference/android/content/Intent

// https://developer.android.com/reference/android/app/PendingIntent

// https://developer.android.com/guide/components/intents-common

// https://developer.android.com/reference/android/content/Intent#resolveActivity(android.content.pm.PackageManager)

@DesignerComponent(version = 1 //
    , versionName = UrsAI2NotificationChannel.VersionName //
    , dateBuilt = UrsAI2NotificationChannel.dateBuilt //
    , description = "Collection of Intent properties." //
    , category = com.google.appinventor.components.common.ComponentCategory.EXTENSION //
    , nonVisible = true //
    , helpUrl = "https://UllisRoboterSeite.de/android-AI2-notifier.html" //
    , iconName = "aiwebres/iconintent.png")
@SimpleObject(external = true)
public class UrsIntent extends AndroidNonvisibleComponent {
  static final String LOG_TAG = ChannelProperties.LOG_TAG;

  int intentType = 0; // 0: None, 1: Screen, 2: Launcher, 3: Event, 4: GoBack
  String intentTypeString = "None";
  String screenToOpen = "Screen1";
  String screenStartValue = "";
  String action = "";
  String dataUri = "";
  YailList extras = new YailList();
  String dataType = "";
  String activityPackage = "";
  String activityClass = "";
  String intentId = "";

  boolean flagClearTask = false;
  boolean flagClearTop = false;
  boolean flagExcludeFromRecents = false;
  boolean flagNewTask = true;
  boolean flagNoAnimation = false;
  boolean flagNoHistory = false;
  boolean flagReorderToFront = false;
  boolean flagPreviousIsTop = false;
  boolean flagSingleTop = false;
  boolean flagResetTaskIfNeeded = false;
  boolean flagTaskOnHome = false;

  public UrsIntent(ComponentContainer container) {
    super(container.$form());
  }

  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_CHOICES //
      , defaultValue = "None" //
      , editorArgs = { "None", "Screen", "Launcher", "Event", "GoBack" })
  @SimpleProperty(description = "Specifies the type of the Intent.", userVisible = false)
  public void ActionType(String value) {
    value = value.toLowerCase().trim();
    if (value.equals("screen")) {
      intentType = 1;
      intentTypeString = "Screen";
    } else if (value.equals("launcher")) {
      intentType = 2;
      intentTypeString = "Launcher";
    } else if (value.equals("event")) {
      intentType = 3;
      intentTypeString = "Event";
    } else if (value.equals("goback")) {
      intentType = 4;
      intentTypeString = "GoBack";
    } else {
      intentType = 0;
      intentTypeString = "None";
    }
  }

  public String ActionType() {
    return intentTypeString;
  }

  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "Screen1")
  @SimpleProperty(description = "Screen to be openend.")
  public void ScreenToOpen(String value) {
    screenToOpen = value;
  }

  @SimpleProperty(description = "Screen to be openend.")
  public String ScreenToOpen() {
    return screenToOpen;
  }

  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "")
  @SimpleProperty(description = "Start value for AI2 apps.")
  public void ScreenStartValue(String value) {
    screenStartValue = value;
  }

  @SimpleProperty(description = "Start value for AI2 apps.")
  public String ScreenStartValue() {
    return screenStartValue;
  }

  // https://developer.android.com/guide/components/intents-common
  // https://developer.android.com/reference/android/content/Intent#setAction(java.lang.String)
  // API Level 1
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "")
  @SimpleProperty(description = "Specifies the action to be executed.")
  public void ActivityAction(String value) {
    action = value;
  }

  @SimpleProperty(description = "Specifies the action to be executed.")
  public String ActivityAction() {
    return action;
  }

  // https://developer.android.com/reference/android/content/Intent#setData(android.net.Uri)
  // API level 1
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "")
  @SimpleProperty(description = "Specifies the data URI that will be used to start the activity.")
  public void DataUri(String value) {
    dataUri = value;
  }

  @SimpleProperty(description = "Specifies the data URI that will be used to start the activity.")
  public String DataUri() {
    return dataUri;
  }

  // https://developer.android.com/reference/android/content/Intent#setType(java.lang.String)
  // API Level 1
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "")
  @SimpleProperty(description = "The explicit MIME data type.")
  public void DataType(String value) {
    dataType = value;
  }

  @SimpleProperty(description = "The explicit MIME data type.")
  public String DataType() {
    return dataType;
  }

  // https://developer.android.com/reference/android/content/Intent#putExtra(java.lang.String,%20android.os.Parcelable)
  // API Level 1
  @SimpleProperty(description = "The list of key-value pairs that will be passed as extra data to the activity.")
  public void Extras(YailList value) {
    extras = value;
  }

  @SimpleProperty(description = "The list of key-value pairs that will be passed as extra data to the activity.")
  public YailList Extras() {
    return extras;
  }

  // https://developer.android.com/reference/android/content/Intent#setPackage(java.lang.String)
  // API Level 4
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "")
  @SimpleProperty(description = "Specifies the package part of the specific component that will be started.")
  public void ActivityPackage(String value) {
    activityPackage = value.trim();
  }

  @SimpleProperty(description = "Returns the package part of the specific component that will be started.")
  public String ActivityPackage() {
    return activityPackage;
  }

  // https://developer.android.com/reference/android/content/Intent#setClassName(android.content.Context,%20java.lang.String)
  // API Level 1
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "")
  @SimpleProperty(description = "Specifies the class part of the specific component that will be started.")
  public void ActivityClass(String value) {
    activityClass = value.trim();
  }

  @SimpleProperty(description = "Returns the class part of the specific component that will be started.")
  public String ActivityClass() {
    return activityClass;
  }

  // ------------------------- Flags --------------------------
  // https://developer.android.com/reference/android/content/Intent#FLAG_ACTIVITY_CLEAR_TASK
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
  @SimpleProperty(description = "Any existing task that would be associated with the activity is cleared before the activity is started.")
  public void FlagClearTask(boolean value) {
    flagClearTask = value;
  }

  @SimpleProperty(description = "Any existing task that would be associated with the activity is cleared before the activity is started.")
  public boolean FlagClearTask() {
    return flagClearTask;
  }

  // https://developer.android.com/reference/android/content/Intent.html#FLAG_ACTIVITY_RESET_TASK_IF_NEEDED
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
  @SimpleProperty(description = "If this activity is either being started in a new task or bringing to the top an existing task, "
      + "then it will be launched as the front door of the task.")
  public void FlagResetTaskIfNeeded(boolean value) {
    flagResetTaskIfNeeded = value;
  }

  @SimpleProperty(description = "If this activity is either being started in a new task or bringing to the top an existing task, "
      + "then it will be launched as the front door of the task.")
  public boolean FlagResetTaskIfNeeded() {
    return flagResetTaskIfNeeded;
  }

  // https://developer.android.com/reference/android/content/Intent#FLAG_ACTIVITY_TASK_ON_HOME
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
  @SimpleProperty(description = "This flag will cause a newly launching task to be placed on top of the current home activity task.")
  public void FlagTaskOnHome(boolean value) {
    flagTaskOnHome = value;
  }

  @SimpleProperty(description = "This flag will cause a newly launching task to be placed on top of the current home activity task.")
  public boolean FlagTaskOnHome() {
    return flagTaskOnHome;
  }

  // https://developer.android.com/reference/android/content/Intent#FLAG_ACTIVITY_SINGLE_TOP
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
  @SimpleProperty(description = "the activity will not be launched if it is already running at the top of the history stack.")
  public void FlagSingleTop(boolean value) {
    flagSingleTop = value;
  }

  @SimpleProperty(description = "the activity will not be launched if it is already running at the top of the history stack.")
  public boolean FlagSingleTop() {
    return flagSingleTop;
  }

  // https://developer.android.com/reference/android/content/Intent#FLAG_ACTIVITY_PREVIOUS_IS_TOP
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
  @SimpleProperty(description = "If set and this intent is being used to launch a new activity from an existing one, "
      + "the current activity will not be counted as the top activity for deciding whether the new intent should be delivered "
      + "to the top instead of starting a new one.")
  public void FlagPreviousIsTop(boolean value) {
    flagPreviousIsTop = value;
  }

  @SimpleProperty(description = "If set and this intent is being used to launch a new activity from an existing one, "
      + "the current activity will not be counted as the top activity for deciding whether the new intent should be delivered "
      + "to the top instead of starting a new one.")
  public boolean FlagPreviousIsTop() {
    return flagPreviousIsTop;
  }

  //https://developer.android.com/reference/android/content/Intent#FLAG_ACTIVITY_REORDER_TO_FRONT
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
  @SimpleProperty(description = "This flag will cause the launched activity to be brought to the front of its task's history stack if it is already running.")
  public void FlagReorderToFront(boolean value) {
    flagReorderToFront = value;
  }

  @SimpleProperty(description = "This flag will cause the launched activity to be brought to the front of its task's history stack if it is already running.")
  public boolean FlagReorderToFront() {
    return flagReorderToFront;
  }

  // https://developer.android.com/reference/android/content/Intent#FLAG_ACTIVITY_CLEAR_TOP
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
  @SimpleProperty(description = "")
  public void FlagClearTop(boolean value) {
    flagClearTop = value;
  }

  @SimpleProperty(description = "")
  public boolean FlagClearTop() {
    return flagClearTop;
  }

  // https://developer.android.com/reference/android/content/Intent#FLAG_ACTIVITY_EXCLUDE_FROM_RECENTS
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
  @SimpleProperty(description = "The launched activity is not kept in the list of recently launched activities.")
  public void FlagExcludeFromRecents(boolean value) {
    flagExcludeFromRecents = value;
  }

  @SimpleProperty(description = "The launched activity is not kept in the list of recently launched activities.")
  public boolean FlagExcludeFromRecents() {
    return flagExcludeFromRecents;
  }

  // https://developer.android.com/reference/android/content/Intent#FLAG_ACTIVITY_MULTIPLE_TASK

  // https://developer.android.com/reference/android/content/Intent#FLAG_ACTIVITY_NEW_DOCUMENT

  // https://developer.android.com/reference/android/content/Intent#FLAG_ACTIVITY_NEW_TASK
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "True")
  @SimpleProperty(description = ".")
  public void FlagNewTask(boolean value) {
    flagNewTask = value;
  }

  @SimpleProperty(description = ".")
  public boolean FlagNewTask() {
    return flagNewTask;
  }

  // https://developer.android.com/reference/android/content/Intent#FLAG_ACTIVITY_NO_ANIMATION
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
  @SimpleProperty(description = "This flag will prevent the system from applying an activity transition animation to go to the next activity state.")
  public void FlagNoAnimation(boolean value) {
    flagNoAnimation = value;
  }

  @SimpleProperty(description = "This flag will prevent the system from applying an activity transition animation to go to the next activity state.")
  public boolean FlagNoAnimation() {
    return flagNoAnimation;
  }

  // https://developer.android.com/reference/android/content/Intent#FLAG_ACTIVITY_NO_HISTORY
  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_BOOLEAN, defaultValue = "False")
  @SimpleProperty(description = "The launched activity is not kept in the history stack.")
  public void FlagNoHistory(boolean value) {
    flagNoHistory = value;
  }

  @SimpleProperty(description = "The launched activity is not kept in the history stack.")
  public boolean FlagNoHistory() {
    return flagNoHistory;
  }

  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_STRING, defaultValue = "")
  @SimpleProperty(description = "Specifies ID for Intents of Type 'Event'.")
  public void ID(String value) {
    intentId = value;
  }

  @SimpleProperty(description = "Specifies ID for Intents of Type 'Event'.")
  public String ID() {
    return intentId;
  }

  int getFlags() {
    int flags = 0;
    if (flagClearTask)
      flags += Intent.FLAG_ACTIVITY_CLEAR_TASK;
    if (flagClearTop)
      flags += Intent.FLAG_ACTIVITY_CLEAR_TOP;
    if (flagExcludeFromRecents)
      flags += Intent.FLAG_ACTIVITY_EXCLUDE_FROM_RECENTS;
    if (flagNewTask)
      flags += Intent.FLAG_ACTIVITY_NEW_TASK;
    if (flagNoAnimation)
      flags += Intent.FLAG_ACTIVITY_NO_ANIMATION;
    if (flagNoHistory)
      flags += Intent.FLAG_ACTIVITY_NO_HISTORY;
    if (flagReorderToFront)
      flags += Intent.FLAG_ACTIVITY_REORDER_TO_FRONT;
    if (flagPreviousIsTop)
      flags += Intent.FLAG_ACTIVITY_PREVIOUS_IS_TOP;
    if (flagSingleTop)
      flags += Intent.FLAG_ACTIVITY_SINGLE_TOP;
    if (flagResetTaskIfNeeded)
      flags += Intent.FLAG_ACTIVITY_RESET_TASK_IF_NEEDED;
    if (flagTaskOnHome)
      flags += Intent.FLAG_ACTIVITY_TASK_ON_HOME;

    flags += Intent.FLAG_GRANT_READ_URI_PERMISSION;

    return flags;
  }

  // TODO: Kann bei Kodular zu problemen führen
  public static Uri getPackageUri(Context context, File apk) {
    if (Build.VERSION.SDK_INT < Build.VERSION_CODES.N) {
      // Use file: URI on versions of Android older the Nougat
      return Uri.fromFile(apk);
    } else {
      // File provider used for SDK 24+ to get a content: URI
      String packageName = context.getPackageName();
      return FileProvider.getUriForFile(context, packageName + ".provider", apk);
    }
  }

  // Aus lokalen Variablen
  PendingIntent buildIntent(int numberID) {
    return build(form, intentType, screenToOpen, screenStartValue, action, dataUri, extras, dataType, activityPackage,
        activityClass, getFlags(), numberID, intentId, form.getClass().getPackage().getName());
  }

  static PendingIntent build(Context context, int intentType, String screenToOpen, String screenStartValue,
      String action, String dataUri, YailList extras, String dataType, String activityPackage, String activityClass,
      int flags, int numberID, String intentID, String formPackage) {

    if (intentType == 0) // None
      return null; // leerer Intent

    // Führt dazu, dass die Notification-Seite geschlossen und die letzte App wieder angezeigt wird
    if (intentType == 4) // GoBack
      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M)
        return PendingIntent.getActivity(context, 0, new Intent(),
            PendingIntent.FLAG_UPDATE_CURRENT | PendingIntent.FLAG_IMMUTABLE);
      else
        return PendingIntent.getActivity(context, 0, new Intent(), PendingIntent.FLAG_UPDATE_CURRENT);

    if (intentType == 3) { // Event auslösen
      Intent intent = new Intent(NotificationProperties.EVENT_TAG);
      intent.putExtra(NotificationProperties.ID_TAG, numberID);
      intent.putExtra(NotificationProperties.EVENT_TAG, intentID);

      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M)
        return PendingIntent.getBroadcast(context, 0, intent,
            PendingIntent.FLAG_UPDATE_CURRENT | PendingIntent.FLAG_IMMUTABLE);
      else
        return PendingIntent.getBroadcast(context, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT);
    }

    Intent intent = new Intent();

    // --- Ziel hinzufügen
    if (intentType == 1) { // Screen
      intent.setComponent(new ComponentName(context.getPackageName(), formPackage + "." + screenToOpen));
    } else {// Launcher
      if (activityPackage.length() != 0 || activityClass.length() != 0) {
        intent.setComponent(new ComponentName(activityPackage, activityClass));
      }
    }

    // --- Action hinzufügen
    intent.setAction(action);

    // --- Extras hinzufügen, aus ActivityStarter entnommen
    // If the extra value is a string, put it to the intent. If the extra value is a
    // list
    // of strings, convert it to a java list and put that to the intent.
    for (Object extra : extras.toArray()) {
      YailList castExtra = (YailList) extra;
      String key = castExtra.getString(0);
      Object value = castExtra.getObject(1);
      if ((key.length() != 0)) {
        if (value instanceof YailList) {
          intent.putExtra(key, ((YailList) value).toStringArray());
        } else {
          String stringValue = castExtra.getString(1);
          intent.putExtra(key, stringValue);
        }
      }

    }

    if (!screenStartValue.isEmpty()) {
      intent.putExtra("APP_INVENTOR_START", screenStartValue);
    }

    // --- Uri & MIME-Type hinzufügen, aus ActivityStarter entnommen
    Uri uri = (dataUri.length() != 0) ? Uri.parse(dataUri) : null;
    if (uri != null && dataUri.toLowerCase().startsWith("file://")) {
      File file = new File(uri.getPath());
      if (file.isFile()) {
        uri = getPackageUri(context, file);
      }
    }

    if (dataType.length() != 0) {
      if (uri != null) {
        intent.setDataAndType(uri, dataType);
      } else {
        intent.setType(dataType);
      }
    } else {
      intent.setData(uri);
    }

    // --- Flags hinzufügen
    intent.setFlags(flags);

    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M)
      return PendingIntent.getActivity(context, 0, intent,
          PendingIntent.FLAG_UPDATE_CURRENT | PendingIntent.FLAG_IMMUTABLE);
    else
      return PendingIntent.getActivity(context, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT);
  }

  public Intent addPropertiesToIntent(Intent intent, int requestCode) {
    intent.putExtra("RequestCode", requestCode);
    intent.putExtra("IntentType", intentType);
    intent.putExtra("IntentAction", action);
    intent.putExtra("IntentScreenToOpen", screenToOpen);
    intent.putExtra("IntentStartValue", screenStartValue);
    intent.putExtra("IntentDataUri", dataUri);
    intent.putExtra("IntentDataType", dataType);
    intent.putExtra("IntentPackage", activityPackage);
    intent.putExtra("IntentClass", activityClass);
    intent.putExtra("IntentFlags", getFlags());
    intent.putExtra("IntentExtras", extras.toJSONString());
    intent.putExtra("IntentFormPackage", form.getClass().getPackage().getName());
    intent.putExtra(NotificationProperties.EVENT_TAG, intentId);
    return intent;
  }

  static PendingIntent buildFromIntentExtras(Context context, Intent dataIntent) {
    int intentType = dataIntent.getIntExtra("IntentType", 0);
    String action = dataIntent.getStringExtra("IntentAction");
    String activityPackage = dataIntent.getStringExtra("IntentPackage");
    String activityClass = dataIntent.getStringExtra("IntentClass");
    String screenToOpen = dataIntent.getStringExtra("IntentScreenToOpen");
    String screenStartValue = dataIntent.getStringExtra("IntentStartValue");
    String dataUri = dataIntent.getStringExtra("IntentDataUri");
    String dataType = dataIntent.getStringExtra("IntentDataType");
    int flags = dataIntent.getIntExtra("IntentFlags", 0);
    int requestCode = dataIntent.getIntExtra("RequestCode", 0);
    int notificationId = dataIntent.getIntExtra(NotificationProperties.ID_TAG, -8888);
    String intentId = dataIntent.getStringExtra(NotificationProperties.EVENT_TAG);
    String formPackage = dataIntent.getStringExtra("IntentFormPackage");
    if (intentId == null)
      intentId = "";
    YailList extras = YailList
        .makeList((List) JsonUtil.getObjectFromJson(dataIntent.getStringExtra("IntentExtras"), false));
    //TODO: Es fehlen die Actions
    return build(context, intentType, screenToOpen, screenStartValue, action, dataUri, extras, dataType,
        activityPackage, activityClass, flags, notificationId, intentId, formPackage);

  }
}